﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using ITIN.CSharp.Types;

namespace ITIN.CSharp.BaseObjects
{
    /// <summary>
    /// Класс, счета, привязанный к бирже(площадке)
    /// </summary>
    public class Account
    {
        #region Fields
        
        private string _name;              // наименование счета (номер)
        private Exchange _exchange;        // ссылка на экземпляр биржи
        private PortfolioStatus _status;   // статус счета
        private double _cash;              // сумма доступных средств на счете для торговли на площадке _exchange
        private double _leverage;          // размер плеча для маржинальной торговли
        private double _comission;         // сумма биржевой комиссии за торговую сессию
        private double _saldo;             // Сальдо (торговый результат) за сессию

        #endregion Fields
        
        #region Methods

        /// <summary>
        /// Конструктор Счета(портфеля)
        /// </summary>
        /// <param name="name">Имя</param>
        /// <param name="exchange">Биржа</param>
        /// <param name="status">Статус</param>
        public Account(string name, Exchange exchange, PortfolioStatus status)
        {
            this._name = name;
            this._exchange = exchange;
            this._status = status;

            this._cash = 0.0;
            this._leverage = 0.0;
            this._comission = 0.0;
            this._saldo = 0.0;
        }

        /// <summary>
        /// Обновить сведения о счете(портфеле)
        /// </summary>
        /// <param name="cash">Сумма доступных средств</param>
        /// <param name="leverage">Размер плеча для маржинальной торговли</param>
        /// <param name="comission">Сумма биржевой комиссии за торговую сессию</param>
        /// <param name="saldo">Сальдо (торговый результат) за сессию</param>
        public void UpdatePortfolio(double cash, double leverage, double comission, double saldo)
        {
            this._cash = cash;
            this._leverage = leverage;
            this._comission = comission;
            this._saldo = saldo;
        }

        #endregion Methods

        #region Properties

        /// <summary>
        /// Сумма доступных наличных средств на счёте
        /// </summary>
        public double Cash { get { return this._cash; } }

        /// <summary>
        /// Сумма биржевой комиссии за торговыую сессию
        /// </summary>
        public double Comission { get { return this._comission; } }

        /// <summary>
        /// Плечо для маржинальной торговли
        /// </summary>
        public double Leverage { get { return this._leverage; } }

        /// <summary>
        /// Наименование(номер) счета(портфеля)
        /// </summary>
        public string Name { get { return this._name; } }

        /// <summary>
        /// Биржа (портфель привязываеться к бирже)
        /// </summary>
        public Exchange PortfolioExchange { get { return this._exchange; } }

        /// <summary>
        /// Статус Счета
        /// </summary>
        public PortfolioStatus PortfolioStatus { get { return this._status; } set { this._status = value; } }

        /// <summary>
        /// Сальдо(результат) за текущую торговую сессию
        /// </summary>
        public double Saldo { get { return this._saldo; } }

        #endregion Properties

        #region Interface Implementation

        /// <summary>
        /// Строковое представление экземпляра класса
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {

            return "[" + this._name + ":" + _exchange.Name + "] " + _status.ToString() + "; Cash=" + this._cash.ToString()
                + "; Leverage=" + this._leverage.ToString() + "; Comission=" + this._comission + "; Saldo=" + this._saldo.ToString();
        }

        #endregion Interface Implementation

    }
}
