﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using ITIN.CSharp.Types;

namespace ITIN.CSharp.BaseObjects
{
    /// <summary>
    /// Класс, представляющий исторические данные торгового инструмента
    /// </summary>
    public class Bar
    {
        #region Fields

        private string _code;
        private DateTime _time;
        private double _open;
        private double _high;
        private double _low;
        private double _close;
        private double _volume;

        #endregion fields

        #region Methods

        /// <summary>
        /// Конструкто бара
        /// </summary>
        /// <param name="code">Код бумаги(Symbol)</param>
        /// <param name="time">Время бара</param>
        /// <param name="open">Цена открытия</param>
        /// <param name="high">Максимальная цена</param>
        /// <param name="low">Минимальная цена</param>
        /// <param name="close">Цена закрытия</param>
        /// <param name="volume">Объем</param>
        public Bar(string code, System.DateTime time, double open, double high, double low, double close, double volume)
        {
            this._code = code;
            this._time = time;
            this._open = open;
            this._high = high;
            this._low = low;
            this._close = close;
            this._volume = volume;
        }

        /// <summary>
        /// Метод получения значения поля бара
        /// </summary>
        /// <param name="type">Тип поля</param>
        /// <returns>Значение цены указанного типа параметра бара</returns>
        public double GetBy(BarType type)
        {
            switch (type)
            {
                case BarType.Open:  return this._open;
                case BarType.High:  return this._high;
                case BarType.Low:   return this._low;
                case BarType.Close: return this._close;
                case BarType.AVG:   return ((this._low + this._high) / 2.0);
            }
            return 0.0;
        }
        
        #endregion Methods

        #region Properties

        
        /// <summary>
        /// Индексатор, добавили дополнительны сопсоб обратиться к полю бара
        /// </summary>
        /// <param name="type">Тип поля</param>
        /// <returns></returns>
        public double this[BarType type] { get { return GetBy(type); } }

        /// <summary>
        /// Код бумаги (торгового инструмента)
        /// </summary>
        public string Code { get { return this._code; } }
        
        /// <summary>
        /// Время
        /// </summary>
        public DateTime Time { get { return this._time; } }
        
        /// <summary>
        /// Цена открытия бара
        /// </summary>
        public double Open { get { return this._open; } }
        
        /// <summary>
        /// Максимальная цена бара
        /// </summary>
        public double High { get { return this._high; } }
        
        /// <summary>
        /// Минимальная цена бара
        /// </summary>
        public double Low { get { return this._low; } }
        
        /// <summary>
        /// Цена закрытия
        /// </summary>
        public double Close { get { return this._close; } }

        /// <summary>
        /// Проторгованое количество бумаг на текущем интервале бара
        /// </summary>
        public double Volume { get { return this._volume; } }

        #endregion Properties

        #region InterfaceImplementation

        public override string ToString()
        {
            return "Bar[" + _code + "] " + _time.ToString() + " Open:" + _open + " High:" + _high + " Low:" + _low + " Close:" + _code + " Volume:" + _volume;
        }

        #endregion InterfaceImplementation
    }
}
