﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using ITIN.CSharp.Types;

namespace ITIN.CSharp.BaseObjects
{
    /// <summary>
    /// Класс сделки
    /// </summary>
    public class Deal
    {
        #region Methods
        
        /// <summary>
        /// Конструктор сделки
        /// </summary>
        /// <param name="account">Счет</param>
        /// <param name="ticker">Инструмент</param>
        /// <param name="price">Цена</param>
        /// <param name="amount">Количество в сделке</param>
        /// <param name="dealId">Идентификатор сделки</param>
        /// <param name="time">Время сделки</param>
        /// 

        
        public Deal(Account account, Ticker ticker, 
            double price, double amount, 
            string dealId, DateTime time) 
        {
            this.Account = account;
            this.Ticker = ticker;
            this.Price = price;            
            this.Amount = amount;
            this.DealId = dealId;
            this.Time = time;
        }
        
        #endregion Methods


        #region Properties

        /// <summary>
        /// Счет
        /// </summary>
        public Account Account { get; private set; }


        /// <summary>
        /// Инструмент
        /// </summary>
        public Ticker Ticker { get; private set; }

        /// <summary>
        /// Цена
        /// </summary>
        public double Price { get; private set; }

        /// <summary>
        /// Количество (объем) в сделке
        /// </summary>
        public double Amount { get; private set; }

        /// <summary>
        /// Идентфикатор сделки
        /// </summary>
        public string DealId { get; private set; }

        /// <summary>
        /// Время сделки
        /// </summary>
        public DateTime Time { get; private set; }

        /// <summary>
        /// Направление сделки
        /// </summary>
        public Side Side
        {
            get
            {
                if (this.Amount < 0.0) return Types.Side.Sell;
                return Types.Side.Buy;
            }
        }

        #endregion Properties

    }
}
