﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ITIN.CSharp.BaseObjects
{
    /// <summary>
    /// Класс, представляющий биржу с набором инструментов
    /// </summary>
    public class Exchange
    {
        #region Fields

        private string _name;                            // наименование биржи
        private SortedList<string, Ticker> _instruments; // набор инструментов

        #endregion Fields

        #region Methods
        
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="name">Имя биржи</param>
        public Exchange(string name)
        {
            this._name = name;
            this._instruments = new SortedList<string, Ticker>();
            //this._account = new Account();
        }

        /// <summary>
        /// Добавить инструмент к текущей бирже
        /// </summary>
        /// <param name="instrumentName">Имя инструмента</param>
        /// <param name="ticker"></param>
        public void Add(string symbol, Ticker ticker)
        {
            if (this._instruments.ContainsKey(symbol) == false)
            {
                ticker.SetExchange(this);
                this._instruments.Add(symbol, ticker);
            }
        }

        #endregion Methods

        #region Properties

        /// <summary>
        /// Наименование биржи
        /// </summary>
        public string Name { get { return this._name;} }

        /// <summary>
        /// Индексатор 
        /// </summary>
        /// <param name="tickerName">Наименование тикера</param>
        /// <returns></returns>
        public Ticker this[string tickerName]
        {
            get
            {
                if (_instruments.ContainsKey(tickerName) == true)
                {
                    return _instruments[tickerName];
                }
                return null;
            }
        }
        
        #endregion Properties

        #region Interface Implementation

        /// <summary>
        /// Возвращает строковое представление 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this._name + " of [" + _instruments.Count + "] tickers ";
        }

        #endregion Interface Implementation
    }
}
