﻿using System;
using System.Collections.Generic;
using System.Collections.Concurrent;
using System.Linq;
using System.IO;
using System.Threading;


namespace ITIN.CSharp.BaseObjects
{
    /// <summary>
    /// Класс- помощник, для ведения логов
    /// </summary>
    public class Logger
    {
        #region Fields

        /// <summary>
        /// Статический экземпляр класса
        /// </summary>
        private static Logger _Logger = new Logger();

        /// <summary>
        /// файл вывода логов
        /// </summary>
        StreamWriter m_logFile;
        
        /// <summary>
        /// Очередь сообщений для протоколирования
        /// </summary>
        private ConcurrentQueue<string> _QueueSend = new ConcurrentQueue<string>();
        
        /// <summary>
        /// Событие прихода новых данных в очередь
        /// </summary>
        private AutoResetEvent _ARE_Send = new AutoResetEvent(false);

        #endregion Fields

        #region Methods

        /// <summary>
        /// Конструктор 
        /// </summary>
        private Logger()
        {
            if (m_logFile == null)
            {
                m_logFile = new StreamWriter("LOG.log", false, System.Text.Encoding.Unicode);
            }

            ThreadPool.QueueUserWorkItem(delegate
            {
                Thread.CurrentThread.Name = "Logger Thread";
                this.LOG_Thread();
            });
        }

        /// <summary>
        /// Метод получения данных из очереди (работает в отдельном потоке)
        /// </summary>
        private void LOG_Thread()
        {
            string log = null;
            while (true)
            {
                if (this._QueueSend.Count == 0) _ARE_Send.WaitOne(300);
                if (this._QueueSend.TryDequeue(out log) == false) continue;

                if (log.ToString().Contains("&"))
                {
                    log = log.ToString().Remove(0, 1);
                    _Logger.m_logFile.WriteLine(log);
                    _Logger.m_logFile.Flush();
                    System.Diagnostics.Debug.WriteLine(log);
                }
                else if (log.ToString().Contains("#"))
                {
                    log = log.ToString().Remove(0, 1);
                    _Logger.m_logFile.WriteLine(log);
                    _Logger.m_logFile.Flush();

                }
                else
                {
                    _Logger.m_logFile.Write(log);
                    _Logger.m_logFile.Flush();
                }
            }
        }

        /// <summary>
        /// Протоколирует строку с сообщением в окне Output
        /// </summary>
        /// <param name="s">Сообщение</param>
        public static void LogWriteLine(object s)
        {
            //lock (_Logger._QueueSend)
            System.Threading.Monitor.Enter(_Logger._QueueSend);
            _Logger._QueueSend.Enqueue("&" + DateTime.Now.ToLongTimeString() + " " + s.ToString());
            _Logger._ARE_Send.Set();
        }

        /// <summary>
        /// Протоколирует строку с сообщением в окне Output и в файле
        /// </summary>
        /// <param name="s">Сообщение</param>
        public static void LogFileWriteLine(object s)
        {
            _Logger._QueueSend.Enqueue("#" + DateTime.Now.ToLongTimeString() + " " + s.ToString());
            _Logger._ARE_Send.Set();
        }

        /// <summary>
        /// Протоколирует сообщение в файле
        /// </summary>
        /// <param name="s">Сообщение</param>
        public static void FileWrite(object s)
        {
            _Logger._QueueSend.Enqueue(s.ToString());
            _Logger._ARE_Send.Set();
        }

        #endregion Fields
    }
}
