﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

//using SmartCOM3Lib;

namespace ITIN.CSharp.BaseObjects
{
    /// <summary>
    /// Общие котировочные данные по инструменту
    /// </summary>
    public class QuoteCommon
    {
        #region Fields
        
        /// <summary>
        /// Tикер инструмента
        /// </summary> 
        private Ticker _ticker;

        /// <summary>
        /// Верхний лиит цены
        /// </summary>
        private double _highLimit;
        
        /// <summary>
        /// Нижний лимит цены
        /// </summary>
        private double _lowLimit;

        /// <summary>
        /// Открытые позиции
        /// </summary>
        private double _openPosition;

        /// <summary>
        /// Гарантийное обеспечение покупателя
        /// </summary>
        private double _goBuy;

        /// <summary>
        /// Гарантийное обеспечение продавца
        /// </summary>
        private double _goSell;
        
        /// <summary>
        /// Гарантийное обеспечение базовое (для продажи опционов)
        /// </summary>
        private double _goBase;
        
        /// <summary>
        /// Гарантийное обеспечение по синтетическим позициям (опционы) 
        /// </summary>
        private double _goBaseBacked;

        #endregion Fields

        #region Methods

        /// <summary>
        /// Конструктор класса QuoteCommon (общие данные по инструменту)
        /// </summary>
        /// <param name="ticker">инструмент</param>
        public QuoteCommon(Ticker ticker)
        {
            this._ticker = ticker;
            UpdateQuote(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
        }

        /// <summary>
        /// Обновить общую информацию
        /// </summary>
        /// <param name="highLimit">верхний лимит цены</param>
        /// <param name="lowLimit">нижний лимит цены</param>
        /// <param name="openPosition">открытые позиции</param>
        /// <param name="goBuy">ГО покупателя</param>
        /// <param name="goSell"><ГО продавца/param>
        /// <param name="goBase">ГО базовое (продажа опцинов)</param>
        /// <param name="goBaseBacked">ГО по синтетически позициям (опционы)</param>
        public void UpdateQuote(
            double highLimit, double lowLimit, double openPosition, 
            double goBuy, double goSell, double goBase, double goBaseBacked)
        {
            this._highLimit = highLimit;
            this._lowLimit = lowLimit;
            this._openPosition = openPosition;
            this._goBuy = goBuy;
            this._goSell = goSell;
            this._goBase = goBase;
            this._goBaseBacked = goBaseBacked;
        }

        #endregion Methods

        #region Properties
        
        /// <summary>
        /// Инструмент
        /// </summary>
        public Ticker Ticker { get { return this._ticker; } }

        /// <summary>
        /// Верхний предел цены
        /// </summary>
        public double HighLimit { get { return this._highLimit; } }

        /// <summary>
        /// Нижний предел цены
        /// </summary>
        public double LowLimit { get { return this._lowLimit; } }

        /// <summary>
        /// Открытые позиции
        /// </summary>
        public double OpenPosition { get { return this._openPosition; } }

        /// <summary>
        /// ГО продавца
        /// </summary>
        public double GoBuy { get { return this._goBuy; } }

        /// <summary>
        /// ГО Покупателя
        /// </summary>
        public double GoSell { get { return this._goSell; } }

        /// <summary>
        /// Базовое значение ГО для продажи опционов
        /// </summary>
        public double GoBase { get { return this._goBase; } }

        /// <summary>
        /// ГО по синтетическим позициям (опционы) 
        /// </summary>
        public double GoBaseBacked { get { return this._goBaseBacked; } }

        #endregion Properties

    }

}
