﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ITIN.CSharp.Types
{
    /// <summary>
    /// Статус Сервера
    /// </summary>
    public enum ServerStatus
    {
        /// <summary>
        /// Сервер доступен
        /// </summary>
        Available,

        /// <summary>
        /// Сервер не доступен
        /// </summary>
        NotAvailable,
    }

    /// <summary>
    /// Состояние соединения
    /// </summary>
    public enum ConnectionState
    {
        /// <summary>
        /// Соединение разорвано
        /// </summary>
        Disconnected,

        /// <summary>
        /// Соединение установлено
        /// </summary>
        Connected
    }

    /// <summary>
    /// Перечисляет способы провержи жизнеспособности сервера
    /// </summary>
    public enum AliveServerMethod
    {
        /// <summary>
        /// Проверка сервера пингом
        /// </summary>
        TryPing,

        /// <summary>
        /// Попытка соединиться с открытым портом сервера
        /// </summary>
        TryPortOpen,
    }

    /// <summary>
    /// Тип значения исторических данных (тип параметра бара)
    /// </summary>
    public enum BarType
    {
        /// <summary>
        /// Цена открытия бара
        /// </summary>
        Open,
        
        /// <summary>
        /// Максимальное значение цены на интервале бара
        /// </summary>
        High,

        /// <summary>
        /// Минимальное значение цены на интервале бара
        /// </summary>
        Low,

        /// <summary>
        /// Цена закрытия бара
        /// </summary>
        Close,

        /// <summary>
        /// Среднее значение
        /// </summary>
        AVG
    }

    /// <summary>
    /// Статус порфеля
    /// </summary>
    public enum PortfolioStatus
    {
        /// <summary>
        /// Брокерский
        /// </summary>
        Broker = 0,
        
        /// <summary>
        /// ДУ
        /// </summary>
        TrustedManagement = 1,
        
        /// <summary>
        /// Котировочный (данные только для просмотра)
        /// </summary>
        ReadOnly = 2,

        /// <summary>
        /// Заблокированный
        /// </summary>
        Blocked = 3,

        /// <summary>
        /// Ограниченный
        /// </summary>
        Restricted = 4,

        /// <summary>
        /// Ограниченный(авто)
        /// </summary>
        AutoRestricted = 5,

        /// <summary>
        /// Ограниченный (реестр)
        /// </summary>
        OrderNotSigned = 6,
    }

    public enum OrderState
    {    
        /// <summary>
        /// Отклонен
        /// </summary>        
        ContragentReject = -1,
        
        /// <summary>
        /// Принят ТС
        /// </summary>
        Submited = 0,
        
        /// <summary>
        /// Зарегистрирован в ТС
        /// </summary>
        Pending = 1,

        /// <summary>
        /// Выведен на рынок
        /// </summary>
        Open = 2,

        /// <summary>
        /// Снят по окончанию торгового дня
        /// </summary>
        Expired = 3, 

        /// <summary>
        /// Отменён
        /// </summary>
        Cancel = 4, 
        
        /// <summary>
        /// Исполнен
        /// </summary>
        Filled = 5,
        
        /// <summary>
        /// Частично исполнен
        /// </summary>
        Partial = 6,
        
        /// <summary>
        /// Отклонён биржей
        /// </summary>
        ContragentCancel = 7,
        
        /// <summary>
        /// Отклонён биржей
        /// </summary>
        SystemReject = 8, 
        
        /// <summary>
        /// Отклонён биржей
        /// </summary>
        SystemCancel = 9,

        /// <summary>
        /// Ордер отправлен
        /// </summary>
        Sending,

        /// <summary>
        /// Ордер Перемещаеться
        /// </summary>
        Moving,

        /// <summary>
        /// Ордер отменяеться
        /// </summary>
        Canceling,

        /// <summary>
        /// Ордер не используется
        /// </summary>
        Disposed
    }

    /// <summary>
    /// Вид торговой операции SmartCOM. Принимает следующие значения:
    /// </summary>
    enum SCOMOrderAction 
    {
        /// <summary>
        /// Купить
        /// </summary>
        Buy = 1,

        /// <summary>
        /// Продать
        /// </summary>
        Sell = 2,

        /// <summary>
        /// Открыть «короткую позицию»
        /// </summary>
        Short = 3,

        /// <summary>
        /// Закрыть «короткую» позицию
        /// </summary>
        Cover = 4
    }
    /// <summary>
    /// Направление сортировки цен в стакане
    /// </summary>
    public enum SortPriceOrder
    {
        /// <summary>
        /// Сортировка по возрастанию
        /// </summary>
        Ascending,
        
        /// <summary>
        /// Сортировка по убыванию
        /// </summary>
        Descending
    }

    /// <summary>
    /// Направление ордера, сделки, позиции (Flat)
    /// </summary>
    public enum Side
    {
        /// <summary>
        /// продажа
        /// </summary>
        Sell = -1,
        
        /// <summary>
        /// нейтральная позиция
        /// </summary>
        Flat = 0,
        
        /// <summary>
        /// покупка
        /// </summary>
        Buy = 1
    }

    /// <summary>
    /// Тип ордера
    /// </summary>
    public enum OrderType
    {
        /// <summary>
        /// Рыночная (встречная) заявка
        /// Срок жизни специфичен для торговой площадки, день или одна торговая сессия)
        /// </summary>
        Market = 1,

        /// <summary>
        /// Лимитная (котировочная) заявка
        /// Срок жизни специфичен для торговой площадки, день или одна торговая сессия)
        /// </summary>
        Limit = 2,
        
        /// <summary>
        /// Стоп ордер
        /// Срок жизни специфичен для торговой площадки, день или одна торговая сессия)
        /// </summary>
        Stop = 3,
        
        /// <summary>
        /// Стоп-лимит ордер
        /// Срок жизни специфичен для торговой площадки, день или одна торговая сессия)
        /// </summary>
        StopLimit = 4,

        /// <summary>
        /// Лимит ордер с указываемым сроком жизни
        /// Срок жизни специфичен для торговой площадки
        ///  SmartCOM не более 30)
        /// </summary>
        LimitGTD = 5,

        /// <summary>
        /// Стоп-лимит ордер с указываемым сроком жизни
        /// Срок жизни специфичен для торговой площадки
        /// ( SmartCOM не более 30)
        /// </summary>
        StopLimitGTD = 6,
    }

    /// <summary>
    /// Команды управления (изменения заявки)
    /// </summary>
    public enum OrderCommand
    {
        // Команды управления
        
        /// <summary>
        /// Отправка заявки
        /// </summary>
        Send,

        /// <summary>
        /// Перемещение
        /// </summary>
        Move,

        /// <summary>
        /// Отмена
        /// </summary>
        Cancel,

        
        // Команды изменения

        /// <summary>
        /// Ордер принят промежуточным звеном (сервером брокера)
        /// </summary>
        Pending,
        
        /// <summary>
        /// Ордер принят торговой площадкой (биржей)
        /// </summary>
        Opened,
        
        /// <summary>
        /// Ордер не выставлен
        /// </summary>
        Failed,
        
        /// <summary>
        /// Ордер перемещен
        /// </summary>
        Moved,

        /// <summary>
        /// Ордер не удалос переместить
        /// </summary>
        MoveFailed,

        /// <summary>
        /// Ордер отменён
        /// </summary>
        Canceled,

        /// <summary>
        /// Не удалось отменить ордер
        /// </summary>
        CancelFailed,

        /// <summary>
        /// Частичное или полное исполнение ордера
        /// </summary>
        OrderFill,

        /// <summary>
        /// Ошибка по ордеру
        /// </summary>
        Error,

        /// <summary>
        /// Ордер освобожден
        /// </summary>
        OrderFree
    }


    /// <summary>
    /// Данные, действия или уведомления, посылаемые в поток обработки событий стратегии
    /// </summary>
    public enum StrategyDataType
    {
        /// <summary>
        /// Данные об изменении "стакана" (очереди заявок)
        /// </summary>
        DOMUpdate,
        
        /// <summary>
        /// Периодическое событие (событие Таймера)
        /// </summary>
        OnTimer,


        /// <summary>
        /// Сделка по инструменту (имеет смысл уточнять - реагировать на свои сделки или нет)
        /// </summary>
        Deal,

        /// <summary>
        /// Своя сделка (частичное исполнение ордера)
        /// </summary>
        OrderPartialFilled,

        /// <summary>
        /// Своя сделка (полное исполнение ордера)
        /// </summary>
        OrderFilled,
        
        /// <summary>
        /// Ордер принят промежуточной системой (брокером)
        /// </summary>
        OrderPending,

        /// <summary>
        /// Ордер зарегистрирован в торговой системе биржи
        /// </summary>
        OrderOpened,

        /// <summary>
        /// Ордер не удалос разместить
        /// </summary>
        OrderFailed,

        /// <summary>
        /// Ордер отмене
        /// </summary>
        OrderCanceled,

        /// <summary>
        /// Уведомление о неудачном снятии заявки
        /// </summary>
        OrderFailCanceled,
        
        /// <summary>
        /// Ордер перемещён
        /// </summary>
        OrderMoved,

        /// <summary>
        /// Уведомление о неудачном перемещении заявки
        /// </summary>
        OrderFailMoved,


        /// <summary>
        /// Уведомление о необходимости выполнить Внешние Action действие 
        /// </summary>
        SGH
    }
}
