﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

using ITIN.CSharp.Types;

namespace ITIN.CSharp.SmartCOM
{
    /// <summary>
    /// Класс экземпляра SmartCOM сервера
    /// </summary>
    public class SmartCOMServerConnectionParametrs : INotifyPropertyChanged
    {
        public bool _isAvailable;
        
        #region SmartCOMServerConnection Properties

        /// <summary>
        /// IP Адрес SmartCOM сервера
        /// </summary>
        public string Ip { get; set; }

        /// <summary>
        /// Порт SmartCOM сервера
        /// </summary>

        /// </summary>
        public short Port { get; set; }

        /// <summary>
        /// Наименование SmartCOM сервера
        /// </summary>
        public string Name { get; set; }


        public string Login { get; set; }
        public string Password { get; set; }

        /// <summary>
        /// Доступность SmartCOM сервера
        /// </summary>
        public bool IsAvailable
        {
            get { return this._isAvailable; }
            set
            {
                if (_isAvailable != value)
                {
                    _isAvailable = value;
                    OnPropertyChanged("IsAvailable");
                }
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Статус SmartCOM сервера
        /// </summary>
        /*public ServerStatus Status {
            get { 
                if (this.IsAvailable == true) {
                    return ServerStatus.Available;
                }
                
                return ServerStatus.NotAvailable;
            }
        }*/

        #endregion SmartCOMServerConnection Properties

        /// <summary>
        /// Конструктор сервера SmartCOM
        /// </summary>
        /// <param name="ip">Адрес сервера</param>
        /// <param name="port">Порт</param>
        /// <param name="name">Наименование</param>
        public SmartCOMServerConnectionParametrs(string ip, short port, string name)
        {
            this.Ip = ip;   // ip адрес
            this.Port = port; // порт
            this.Name = name; // Наименование
            
            this._isAvailable = false;
        }

        /// <summary>
        /// Определяет жизнеспособность сервера
        /// </summary>
        /// <param name="aliveServerMethod">Метод определения статуса сервера</param>
        /// <returns></returns>
        public bool TestOnAvailableSmartComServer(AliveServerMethod aliveServerMethod = AliveServerMethod.TryPing)
        {
            if (aliveServerMethod == AliveServerMethod.TryPing)
            {
                try
                {
                    System.Net.NetworkInformation.Ping ping = new System.Net.NetworkInformation.Ping();
                    System.Net.NetworkInformation.PingReply reply = ping.Send(this.Ip, 1000);

                    bool isAvailable = (reply.Status == (System.Net.NetworkInformation.IPStatus.Success));

                    this.IsAvailable = isAvailable;
                    return isAvailable;
                }
                catch (Exception)
                {
                    return false;
                }
            }
            else
            { // aliveServerMethod == isPingTesting 
                
                System.Net.Sockets.TcpClient tmpSocket = new System.Net.Sockets.TcpClient();
                bool isAvailable = false;
                try
                {
                    tmpSocket.Connect(this.Ip, this.Port);
                    isAvailable = true;

                }
                catch (Exception)
                {
                    isAvailable = false;

                }
                finally
                {
                    if (isAvailable == true)
                    {
                        tmpSocket.Close();
                    }
                }

                this.IsAvailable = isAvailable;

                return isAvailable;
            }
        }

        #region Handlers
        protected void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }
        #endregion Handlers

        #region Interface Implementation
        
        public override string ToString()
        {
            return Name + " - " + Ip;
        }

        #endregion Interface Implementation
    }
}
