﻿using System;
using System.Collections.Concurrent; 
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

using ITIN.CSharp.Types;

namespace ITIN.CSharp.SmartCOM
{
    /// <summary>
    /// Поток торговой системы
    /// </summary>
    internal class EngineWorkerThread
    {
        /// <summary>
        /// События потока торговой системы
        /// </summary>
        internal enum EngineMessage
        {
            /// <summary>
            /// Запуск
            /// </summary>
            StartEngine,
            
            /// <summary>
            /// Останов
            /// </summary>
            StopEngine,
            
            /// <summary>
            /// Соединение с сервером
            /// </summary>
            Connect,

            /// <summary>
            /// Разрыв соединнения
            /// </summary>
            Disconnect
        }

        #region Fields
        
        /// <summary>
        /// Очередь обработки торгового "движок"
        /// </summary>
        private ConcurrentQueue<Action> _actionQueue;

        /// <summary>
        /// Определяет, работает ли поток ядра
        /// </summary>
        private volatile bool _isWork = false;

        /// <summary>
        /// Событие прихода данных в очедерь ядра
        /// </summary>
        private AutoResetEvent _arEvent;

        /// <summary>
        /// Ссылка на "движок"
        /// </summary>
        private TradeEngine _engine;
        
        #endregion Fields
        

        #region Methods

        /// <summary>
        /// Класс потока задданий для "движка"
        /// </summary>
        /// <param name="engine">"Движок"</param>
        internal EngineWorkerThread(TradeEngine engine)
        {
            this._engine = engine;
            this._actionQueue = new ConcurrentQueue<Action>();
            this._arEvent = new AutoResetEvent(false);
            this._isWork = false;
        }

        /// <summary>
        /// Запускает поток опроса очереди заданий
        /// </summary>
        internal void StartEngineThread()
        {
            this._isWork = true;
            while ((this._isWork == true) || (this._actionQueue.Count > 0))
            {
                if (this._actionQueue.Count > 0)
                {
                    Action tmpAction = null;
                    if (_actionQueue.TryDequeue(out tmpAction) == true)
                    {
                        tmpAction();
                    }
                }
                else
                {
                    _arEvent.WaitOne(300);
                }
            }
        }

        /// <summary>
        /// Добавить задание в очередь
        /// </summary>
        /// <param name="task">Задание</param>
        internal void AddEngineTask(Action task)
        {
            this._actionQueue.Enqueue(task);
            this._arEvent.Set();
        }

        /// <summary>
        /// Остановить поток движка
        /// </summary>
        internal void StopEngineThread()
        {
            this._isWork = false;
        }

        #endregion Methods
    }
}
