﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

using ITIN.CSharp.Types;

namespace ITIN.CSharp.SmartCOM
{
    /// <summary>
    /// Класс уведомлений состояний соединения
    /// </summary>
    public class ServerConnectionStateNotifier<TServerConnectionParametrs>
        : INotifyPropertyChanged where TServerConnectionParametrs : class
    {
        #region Fields
        /// <summary>
        /// Состояние соединения
        /// </summary>
        private ConnectionState _state;

        /// <summary>
        /// Подключены ли к серверу
        /// </summary>
        private bool _isConnected;

        /// <summary>
        /// Экземпляр с параметрами соединения
        /// </summary>
        private TServerConnectionParametrs _connection;

        /// <summary>
        /// Событие о изменении свойств (WPF)
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Действие, изменения состояния соединения
        /// </summary>
        public event Action<ConnectionState> ConnectionStateChangeRaised;

        #endregion Fields

        #region Methods

        /// <summary>
        /// Конструктор
        /// </summary>
        public ServerConnectionStateNotifier()
        {
            this._connection = null;
            this._isConnected = false;
            this.CurrentConnectionState = ConnectionState.Disconnected;
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="connection">Внешний экзепмляр параметров соединения</param>
        public ServerConnectionStateNotifier(TServerConnectionParametrs connection)
        {
            this._connection = connection;
            this._isConnected = false;
            this.CurrentConnectionState = ConnectionState.Disconnected;
        }

        #endregion Methods

        #region Properties

        /// <summary>
        /// Состояние соединения
        /// </summary>
        public ConnectionState CurrentConnectionState
        {
            get { return this._state; }
            set
            {
                //var previousConnectionState = this._state;
                //if (previousConnectionState != value)
                {
                    this._state = value;
                    this.OnPropertyChanged("CurrentConnectionState");
                    this.IsConnected = (this._state == ConnectionState.Connected);

                    if (this.ConnectionStateChangeRaised != null)
                    {
                        // если есть подписчики на событие об изменении статуса соединения
                        // вызовем это событие
                        this.ConnectionStateChangeRaised(value);
                    }
                }
                //OnConnectionStatusChange(previousConnectionState, this.State)
            }
        }

        /// <summary>
        /// Выполнено ли подключение к серверу
        /// </summary>
        public bool IsConnected
        {
            get { return this._isConnected; }
            set
            {
                if (this._isConnected != value)
                {
                    this._isConnected = value;
                    OnPropertyChanged("IsConnected");
                }
            }
        }

        /// <summary>
        /// Параметры соединения
        /// </summary>
        public TServerConnectionParametrs Connection
        {
            get { return this._connection; }
            set { this._connection = value; }

        }

        #endregion Properties

        #region Interface implementation
        
        /// <summary>
        /// Уведомление о озменении свойств класса (WPF)
        /// </summary>
        /// <param name="name">Имя изменённого свойства</param>
        protected void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        #endregion Interface implementation
    }
}
