﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


using ITIN.CSharp.BaseObjects;
using ITIN.CSharp.Types;

namespace ITIN.CSharp.Strategy
{
    /// <summary>
    /// Тестовый класс стратегиии.
    /// Рекомендуеться размещать в отдельной DLL
    /// </summary>
    public class TestStrategy : BaseStrategy
    {
        #region Fields

        /// <summary>
        /// Торговая ситема, поставщик данных, обработчик команд управления ордерами
        /// </summary>
        private IEngine _engineData;

        /// <summary>
        /// Стакан покупок
        /// </summary>
        private SnapshotDOM _bids;

        /// <summary>
        /// Стакан продаж
        /// </summary>
        private SnapshotDOM _asks;

        /// <summary>
        /// Счет торговых операций
        /// </summary>
        private Account _tradeAccount;

        /// <summary>
        /// Позиция
        /// </summary>
        private double _position;


        #endregion Fields

        #region Methods

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="engineData">Движок</param>
        /// <param name="name">Имя стратегии</param>
        public TestStrategy(IEngine engineData, string name)
            : base(engineData, name)
        {
            // Обнулим позицию
            this.Position = 0.0;

            this._tradeAccount = null;
            this._engineData = engineData;

            this._bids = new SnapshotDOM();
            this._asks = new SnapshotDOM();
        }

        /// <summary>
        /// Обработчик полцчения сделок
        /// </summary>
        /// <param name="deal">Сделка</param>
        private void DealReceiveRaise(Deal deal)
        {

        }

        /// <summary>
        /// Обработчик изменений стакана
        /// </summary>
        /// <param name="bids">Покупки</param>
        /// <param name="asks">Продажи</param>
        private void DOMUpdatedRaise(SnapshotDOM bids, SnapshotDOM asks)
        {
            //TestSendCancel(bids, asks);                   // Тестирование выставления заявок
            //TestSendMovingCancel(bids, asks);             // Тестирования перемещения и отмены
            //TestSendOrMove(bids, asks);                   // Тестирование выставления и перемещения ордера
            //TestSendMovingCancelInPending(bids, asks);    // Тестирование на ошибки после 22.00
            //TestMoveAndCancelFilledOrder(bids, asks);     // Тестирование управлением не существующих заявок
            TestSellAndExit(bids, asks);                  // Тестирование исполнения, отслеживания позиции и закрытия позиции
        }

        // Тестовые методы

        private void TestSendCancel(SnapshotDOM bids, SnapshotDOM asks)
        {
            if (bids.Count < 2) return;
            TradeOrder enterBuyOrder = base.SendOrder(TradeAccount, TradeTicker, (bids[1].Price - 500.0), 1.0, this.Name + "EnterBuyOrder");
            if (enterBuyOrder == null) return;
            enterBuyOrder.OnPending = (order) => base.CancelOrder(this.Name + "EnterBuyOrder");
            enterBuyOrder.OnOpened = (order) =>
            {
                base.CancelOrder(this.Name + "EnterBuyOrder");
            };
            enterBuyOrder.OnFailOpened = (order, failReason) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось выставить:" + failReason);
            enterBuyOrder.OnFailCanceled = (order) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось отменёнить");
        }

        private void TestSendMovingCancel(SnapshotDOM bids, SnapshotDOM asks)
        {
            if (bids.Count < 2) return;

            TradeOrder enterBuyOrder = base.SendOrder(TradeAccount, TradeTicker, (bids[0].Price - 200.0), 1.0, this.Name + "EnterBuyOrder");
            if (enterBuyOrder == null) return;
            
            enterBuyOrder.OnOpened = (order) =>
            {
                base.MoveOrder(order, (bids[1].Price - 500.0));
                order.OnMoved = (notMovedOrder, oldPrice, newPrice) => CancelOrder(notMovedOrder);

                order.OnFailMoved = (notMovedOrder, oldPrice, newPrice) =>
                    {
                        System.Diagnostics.Debug.WriteLine("Ордер OrderId=" + notMovedOrder.OrderId + ", Name" + notMovedOrder.Name + " не удалось Смувить");
                        CancelOrder(notMovedOrder);
                    };
            };

            enterBuyOrder.OnPending = (order) =>
                {
                    base.MoveOrder(order, (bids[1].Price - 500.0));
                    order.OnMoved = (notMovedOrder, oldPrice, newPrice) => CancelOrder(notMovedOrder);

                    order.OnFailMoved = (notMovedOrder, oldPrice, newPrice) =>
                        {
                            System.Diagnostics.Debug.WriteLine("Ордер OrderId=" + notMovedOrder.OrderId + ", Name" + notMovedOrder.Name + " не удалось Смувить");
                            CancelOrder(notMovedOrder);
                        };
                };

            enterBuyOrder.OnFailOpened = (order, failReason) =>
                {
                    System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось выставить:" + failReason);
                };

            enterBuyOrder.OnFailCanceled = (order) =>
                {
                    System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось отменёнить");
                };
        }

        private void TestSendOrMove(SnapshotDOM bids, SnapshotDOM asks)
        {
            TradeOrder enterBuyOrder = SendOrMoveOrder(TradeAccount, TradeTicker, (bids[1].Price - 500.0), 1.0, 10, this.Name + "EnterBuyOrder");
            if (enterBuyOrder == null) return;
            enterBuyOrder.OnMoved = (order, oldPrice, newPrice) =>
            {
                System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " перемещен с цены" + oldPrice.ToString() + " на цену " + newPrice.ToString());

            };
            //enterBuyOrder.OnPending = (order) => base.CancelOrder(this.Name + "EnterBuyOrder");
            enterBuyOrder.OnOpened = (order) =>
            {
                System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " выставлен по цене " + order.Price.ToString());
            };
            enterBuyOrder.OnFailOpened = (order, failReason) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось выставить" + failReason);
            enterBuyOrder.OnCanceled = (order) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " отменён");
        }

        private void TestSendMovingCancelInPending(SnapshotDOM bids, SnapshotDOM asks)
        {
            if (bids.Count < 2) return;

            TradeOrder enterBuyOrder = base.SendOrder(TradeAccount, TradeTicker, (bids[0].Price - 200.0), 1.0, this.Name + "EnterBuyOrder");
            if (enterBuyOrder == null) return;

            enterBuyOrder.OnPending = (order) =>
            {
                base.MoveOrder(order, (bids[1].Price - 500.0));
                order.OnMoved = (notMovedOrder, oldPrice, newPrice) => CancelOrder(notMovedOrder);

                order.OnFailMoved = (notMovedOrder, oldPrice, newPrice) =>
                {
                    System.Diagnostics.Debug.WriteLine("Ордер OrderId=" + notMovedOrder.OrderId + ", Name" + notMovedOrder.Name + " не удалось Смувить");
                    CancelOrder(notMovedOrder);
                };
            };

            enterBuyOrder.OnFailOpened = (order, failReason) =>
            {
                System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось выставить:" + failReason);
            };

            enterBuyOrder.OnFailCanceled = (order) =>
            {
                System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось отменёнить");
            };
        }

        private void TestMoveAndCancelFilledOrder(SnapshotDOM bids, SnapshotDOM asks)
        {
            if ((bids.Count < 2) || (asks.Count < 2)) return;
            if (this.Position == 0.0)
            {
                TradeOrder enterBuyOrder = SendOrMoveOrder(TradeAccount, TradeTicker, (bids[0].Price - 100.0), 1.0, 10, this.Name + "EnterBuyOrder");
                if (enterBuyOrder == null) return;
                enterBuyOrder.OnOpened= (openedOrder) =>
                {
                    MoveOrder(openedOrder, (bids[0].Price + 200.0));
                    openedOrder.OnMoved = (ord, price1, price2) =>
                        {
                            int d = 0;
                        };
                };
                
                enterBuyOrder.OnPartialFilled = (order, dealAmount) =>
                {
                    this.Position += dealAmount;
                };
                enterBuyOrder.OnFilled = (order) =>
                {
                    MoveOrder(enterBuyOrder, (bids[0].Price) - 100);
                };
            }
            if (this.Position != 0.0)
            {
                TradeOrder exitOrder = SendOrMoveOrder(TradeAccount, TradeTicker, (bids[0].Price - 100.0), -this.Position, 10, this.Name + "exitOrder");
                if (exitOrder == null) return;
                exitOrder.OnPartialFilled = (order, dealAmount) =>
                {
                    this.Position += dealAmount;
                };
                exitOrder.OnFilled = (order) =>
                {
                    CancelOrder(exitOrder);
                };                
            }
        }

        private void TestSellAndExit(SnapshotDOM bids, SnapshotDOM asks)
        {
            if ((bids.Count < 2) || (asks.Count < 2)) return;
            if (this.Position == 0.0)
            {
                TradeOrder enterBuyOrder = SendOrMoveOrder(TradeAccount, TradeTicker, (bids[0].Price), 2.0, 10, this.Name + "EnterBuyOrder");
                if (enterBuyOrder == null) return;
                enterBuyOrder.OnFailOpened = (order, failReason) => CancelOrder(order);
                enterBuyOrder.OnPartialFilled = (order, dealAmount) =>
                {
                    this.Position += dealAmount;
                };
                //enterBuyOrder.OnPending = (order) => base.CancelOrder(this.Name + "EnterBuyOrder");
                enterBuyOrder.OnOpened = (order) =>
                {
                    System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " выставлен по цене " + order.Price.ToString());
                };
                //enterBuyOrder.OnFailOpened = (order, failReason) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось выставить" + failReason);
                enterBuyOrder.OnCanceled = (order) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " отменён");
            }
            else
            {
                TradeOrder exitOrder = SendOrMoveOrder(TradeAccount, TradeTicker, (asks[0].Price - this.TradeTicker.Step), -this.Position, 10, this.Name + "ExitOrder");
                if (exitOrder == null) return;
                exitOrder.OnFailOpened = (order, failReason) => CancelOrder(order);
                exitOrder.OnPartialFilled = (order, dealAmount) =>
                {
                    this.Position += dealAmount;
                };
                //enterBuyOrder.OnPending = (order) => base.CancelOrder(this.Name + "EnterBuyOrder");
                exitOrder.OnOpened = (order) =>
                {
                    //System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " выставлен по цене " + order.Price.ToString());
                };
                //enterBuyOrder.OnFailOpened = (order, failReason) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " не удалось выставить" + failReason);
                //exitOrder.OnCanceled = (order) => System.Diagnostics.Debug.WriteLine("Ордер " + order.Name + " отменён");                
            }
        }
        
        #endregion Methods

        #region Properties

        /// <summary>
        /// Счет
        /// </summary>        
        public Account TradeAccount
        {   get
            {
                if (this._tradeAccount == null)
                {
                    this._tradeAccount = this.Engine.GetAccounts(this.TradeTicker.Exchange)[0];
                }
                return this._tradeAccount;
            }
        }

        /// <summary>
        /// Позиция
        /// Подсчет позиции ведем сами в рамках данной стратегии
        /// В "боевой" системе имеет смысл сделать отдельный класс помощник, для вечисления текущей позиции стратегии по отдельным инсрументам
        /// Также класс долен уметь определять эффективность стратегии и вести учет финансовых показателей
        /// </summary>
        public double Position
        {
            get { return this._position; }
            private set
            {
                this._position = value;
                OnPropertyChanged("Positions");
            }
        }

        /// <summary>
        /// ТОрговый инструмент
        /// </summary>
        public Ticker TradeTicker { get; private set; }

        #endregion Properties

        #region Interface implementation

        /// <summary>
        /// Возвращает позицию стратегии по указанному инструменту
        /// </summary>
        /// <param name="ticker"></param>
        /// <returns>Текущая позиция</returns>
        double GetPosition(Ticker ticker)
        {
            // если стратегия торгует нескольукими инструментами,
            // необходимо по тикеру получить из "менеджера позиции"(отсутствует в данной реализации)
            // текущую позицию по указанному инструменту
            return this.Position;
        }

        /// <summary>
        /// Строка, перечисляющая все позиции
        /// </summary>
        public string Positions
        {
            get
            {
                return this.TradeTicker.Symbol + " : " + this._position.ToString();
            }
        }

        /// <summary>
        /// Запуск стратегии
        /// </summary>
        protected override void OnStarted()
        {
            this.TradeTicker = this.Engine.GetTicker("RTS-6.13_FT");

            this.SubscribeOnDOMUpdate(this.TradeTicker, (bids, asks) =>
                {
                    if (bids != null)
                        BaseObjects.SnapshotDOMExtension.SnapshotDOM.CopyArrayToSnapshot(bids, _bids);
                    
                    if (asks != null)
                        BaseObjects.SnapshotDOMExtension.SnapshotDOM.CopyArrayToSnapshot(asks, _asks);

                    this.DOMUpdatedRaise(_bids, _asks);
                }
            );

            this.SubscribeOnDealReceive(this.TradeTicker, (deal) => this.DealReceiveRaise(deal));
        }

        /// <summary>
        /// Останов Стратегии
        /// </summary>
        protected override void OnStopped()
        {
            Action closePosition = () => 
            {
                ClosePosition(this.TradeTicker,
                    () => 
                        {
                            double price = _bids[0].Price - 15.0 * TradeTicker.Step;
                            if (this.Position > 0 )
                            {
                                price = _asks[0].Price + 15.0 * TradeTicker.Step;
                            }

                            SendOrMoveOrder(TradeAccount, TradeTicker, price, this.Position, 10, "StopStrategyClosePosition");
                        },
                    () =>
                        {
                            base.OnStopped();
                        });
            };
            
            Action strategyAction = () =>
            {
                CancelAllOrder(() => 
                    {
                        closePosition();                        
                    });
            };



            this.SendStrategyData(new object[] { StrategyDataType.SGH, strategyAction });
        }

        #endregion Interface implementation
        
    }
}
