﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;             // CultureInfo
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using ITIN.CSharp.BaseObjects;
using ITIN.CSharp.Strategy;
using ITIN.CSharp.SmartCOM;
using ITIN.CSharp.Types;

namespace ITIN.TradeEngineTestApp
{
    public class ToStartStateConverter : IValueConverter
    {
        /// <summary>
        /// Преобразует значение.
        /// </summary>
        /// <param name="value">Значение, произведенное исходной привязкой.</param>
        /// <param name="targetType">Тип свойства цели связывания.</param>
        /// <param name="parameter">Параметр используемого преобразователя.</param>
        /// <param name="culture">Язык и региональные параметры, используемые в преобразователе.</param>
        /// <returns>Преобразованное значение.Если метод возвращает null, используется действительное значение null.</returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (Int32.Parse(value.ToString()) == 1) return "Started";
            if (Int32.Parse(value.ToString()) == 0) return "Stopped";
            return value.ToString();
        }

        /// <summary>
        /// Преобразует значение.
        /// </summary>
        /// <param name="value">Значение, произведенное целью привязки.</param>
        /// <param name="targetType">Тип, к которому выполняется преобразование.</param>
        /// <param name="parameter">Используемый параметр преобразователя.</param>
        /// <param name="culture">Язык и региональные параметры, используемые в преобразователе.</param>
        /// <returns>Преобразованное значение.Если метод возвращает null, используется действительное значение null.</returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class ToStringConverter : IValueConverter
    {
        /// <summary>
        /// Преобразует значение.
        /// </summary>
        /// <param name="value">Значение, произведенное исходной привязкой.</param>
        /// <param name="targetType">Тип свойства цели связывания.</param>
        /// <param name="parameter">Параметр используемого преобразователя.</param>
        /// <param name="culture">Язык и региональные параметры, используемые в преобразователе.</param>
        /// <returns>Преобразованное значение.Если метод возвращает null, используется действительное значение null.</returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value == null) return "";
            return value.ToString();
        }

        /// <summary>
        /// Преобразует значение.
        /// </summary>
        /// <param name="value">Значение, произведенное целью привязки.</param>
        /// <param name="targetType">Тип, к которому выполняется преобразование.</param>
        /// <param name="parameter">Используемый параметр преобразователя.</param>
        /// <param name="culture">Язык и региональные параметры, используемые в преобразователе.</param>
        /// <returns>Преобразованное значение.Если метод возвращает null, используется действительное значение null.</returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private TradeEngine _engine;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            // Метод добавления текста (для логгера)
            Action<string> onSendMessage = (message) => this.LogMessage(message);
            // Инициализация логгера обработчиком добавления текста из любого потока
            MessageLogger.InitializeLogger((message) =>
                tbxMessages.Dispatcher.BeginInvoke(onSendMessage, message));

            // инициализация торгового движка
            this._engine = new TradeEngine();


            // Свяжем стакан с данными, так делать не рекомендуеться
            this._engine._visualBids = this.vsdDOM.Bids;
            this._engine._visualAsks = this.vsdDOM.Asks;
            this._engine._notifierDOMUpdate = this.vsdDOM;

            // Инициализация списка серверов
            short smartCOMPort = 8090;
            // Сервера, IT Invest            
            //_engine.Servers.Add(new SmartCOMServerConnectionParametrs("95.131.26.246",   smartCOMPort, "Тестовый"));
            _engine.Servers.Add(new SmartCOMServerConnectionParametrs("194.247.135.144", smartCOMPort, "Тестовый"));
            _engine.Servers.Add(new SmartCOMServerConnectionParametrs("62.141.86.229",   smartCOMPort, "Тестовый"));
            _engine.Servers.Add(new SmartCOMServerConnectionParametrs("82.204.220.34",   smartCOMPort, "Основной"));
            _engine.Servers.Add(new SmartCOMServerConnectionParametrs("213.247.232.236", smartCOMPort, "Дополнительный"));
            _engine.Servers.Add(new SmartCOMServerConnectionParametrs("87.118.223.109",  smartCOMPort, "Only RTS"));

            _engine.PingServers();

            this.cbxActiveServer.ItemsSource = _engine.Servers;
            this.cbxActiveServer.SelectedIndex = 0;

            this.tbxLogin.Text = ""; 
            this.tbxPassword.Text = "";

            this.cbxConnectionStatus.IsChecked = false;
            this.cbxConnectionStatus.DataContext = _engine.ConnectionNotifier;

            _engine.ConnectionNotifier.ConnectionStateChangeRaised += new Action<ConnectionState>((newConnectionState) =>
            {
                if (newConnectionState == ConnectionState.Connected)
                {
                    btnConnect.Dispatcher.BeginInvoke(new Action(()=> btnConnect.Content = "Разорвать"), null);
                }
                else
                {
                    btnConnect.Dispatcher.BeginInvoke(new Action(() => btnConnect.Content = "Соединить"), null);
                }
            });

            // потокобезопасное обновление списка стратегий
            _engine.Strategies.ExternalCollectionChangeHandler = (updateCollection) =>
            {
                var values = _engine.Strategies.Values;
                this.itcStrategies.Dispatcher.BeginInvoke(new Action(() =>
                    {
                        var tmpValues = values;
                        this.itcStrategies.ItemsSource = tmpValues;
                        updateCollection();
                    })
                );
            };

            this.itcStrategies.ItemsSource = _engine.Strategies.Values;

            // потокобезопасное обновление списка всех ордеров
            _engine.OwnOrders.GetOrdersDictionary().ExternalCollectionChangeHandler = (updateCollection) =>
            {
                var values = _engine.OwnOrders.GetOrdersDictionary().Values;
                this.itcOwnOrders.Dispatcher.BeginInvoke(new Action(() =>
                    {
                        var tmpValues = values;
                        this.itcOwnOrders.ItemsSource = tmpValues;
                        updateCollection();
                    })
                );
            };

            this.itcOwnOrders.ItemsSource = _engine.OwnOrders.GetOrdersDictionary().Values;

            //
            //var ord = TradeOrder.GetInitializedOrder(null, null, new Ticker("rtyurty", "rttryurt", "rtyutryui", 10.0, 15, 1, "tyutyr", DateTime.Now, 46.0), 150000.0, 1.0, 154);
            //_engine.OwnOrders.AddOrder("1", ord);

        }

        /// <summary>
        /// Обработчик нажатия на кнопку Соединиться/Разорвать
        /// </summary>
        private void btnConnect_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (_engine.IsConnected == true)
                {
                    btnConnect.Content = "Разрываем...";
                }
                else
                {
                    btnConnect.Content = "Соединяем...";
                }

                MessageLogger.LogMessage("Send Connection Action");
                _engine.Connect((SmartCOMServerConnectionParametrs)cbxActiveServer.SelectedValue, tbxLogin.Text, tbxPassword.Text);
            }
            catch (Exception exception)
            {
                MessageLogger.LogMessage("Не удалось установить соединение: " + exception.Message);
            }
        }

        private void btnStartStopStrategies_Click(object sender, RoutedEventArgs e)
        {
            this._engine.StartStopTest();
        }

        private void mnuStartStopStrategy_Click(object sender, RoutedEventArgs e)
        {
            return;
            var strategy = (IStrategy)((MenuItem)sender).Tag;
            if (strategy.IsStarted == true)
            {
                
                strategy.Stop().Wait(20000);                
            }
            else
            {
                strategy.Start();
            }
        }

        private void cbxActiveServer_DropDownOpened(object sender, EventArgs e)
        {
            _engine.PingServers();
        }

        private void LogMessage(string message)
        {
            System.Diagnostics.Debug.WriteLine(message);

            btnConnect.Dispatcher.BeginInvoke(new Action(() =>
                {
                    this.tbxMessages.AppendText(message);
                    this.tbxMessages.AppendText("\r\n");
                    this.tbxMessages.Focus();
                    this.tbxMessages.CaretIndex = this.tbxMessages.Text.Length;
                    this.tbxMessages.ScrollToEnd();
                }));

        }

        private void MenuItem_Click_1(object sender, RoutedEventArgs e)
        {

        }
    }
}
