﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Globalization;             // CultureInfo
using System.Linq;
using System.Text;
//using System.Threading;               // SynchronizationContext
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Threading;         // Dispatcher
using System.Windows.Shapes;


using ITIN.CSharp.BaseObjects;

namespace ITIN.TradeEngineTestApp
{
    

    public class LeveltConverter : IValueConverter
    {
        /// <summary>
        /// Преобразует значение.
        /// </summary>
        /// <param name="value">Значение, произведенное исходной привязкой.</param>
        /// <param name="targetType">Тип свойства цели связывания.</param>
        /// <param name="parameter">Параметр используемого преобразователя.</param>
        /// <param name="culture">Язык и региональные параметры, используемые в преобразователе.</param>
        /// <returns>Преобразованное значение.Если метод возвращает null, используется действительное значение null.</returns>
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            double doubleValue = 0.0;             
            if (Double.TryParse(value.ToString(), out doubleValue) == true)
            {
                if (doubleValue > 0.0) return doubleValue.ToString();
                if (doubleValue < 0.0) return (-doubleValue).ToString();
            }
            return "";
        }
        
        /// <summary>
        /// Преобразует значение.
        /// </summary>
        /// <param name="value">Значение, произведенное целью привязки.</param>
        /// <param name="targetType">Тип, к которому выполняется преобразование.</param>
        /// <param name="parameter">Используемый параметр преобразователя.</param>
        /// <param name="culture">Язык и региональные параметры, используемые в преобразователе.</param>
        /// <returns>Преобразованное значение.Если метод возвращает null, используется действительное значение null.</returns>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
    
    /// <summary>
    /// Interaction logic for VisualDOMSnapshot.xaml
    /// </summary>
    public partial class VisualDOMSnapshot : UserControl, 
        INotifyDOMUpdated, INotifyPropertyChanged
        //INotifyPropertyChanged
    {
        public class VisualShapshot : ObservableCollection<Level>
        {
            #region Fields

            public Dispatcher Dispatcher { get; private set; }

            public Ticker Ticker { get; internal set; }

            public event NotifyCollectionChangedEventHandler _onCollectionChanged;

            #endregion Fields

            public VisualShapshot(Dispatcher dispatcher)
            {
                this.Dispatcher = dispatcher;
                this.Ticker = null;
            }
            
            public void RefreshSnapshot()
            {
                base.OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
                //Dispatcher.BeginInvoke(new Action<NotifyCollectionChangedEventArgs>((nea) => base.OnCollectionChanged(nea)), null);
            }

            protected void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
            {
                Dispatcher.BeginInvoke(new Action<NotifyCollectionChangedEventArgs>((nea) => base.OnCollectionChanged(nea)), e);
            }

            protected void OnPropertyChanged(PropertyChangedEventArgs e)
            {
                Dispatcher.BeginInvoke(new Action<PropertyChangedEventArgs>((pcea) => base.OnPropertyChanged(pcea)), e);
                //Dispatcher.Post((SendOrPostCallback)delegate { base.OnPropertyChanged(e); }, e);
            }
        }


        #region Fields

        private Ticker _ticker;

        #endregion Fields

        #region Methods

        public VisualDOMSnapshot()
        {
            InitializeComponent();

            this._ticker = null;
            this.Asks = new VisualShapshot(this.Dispatcher);
            this.Bids = new VisualShapshot(this.Dispatcher);
        }
        #endregion Methods

        #region Properties

        public VisualShapshot Bids { get; private set; }
        public VisualShapshot Asks { get; private set; }

        //public ObservableCollection<Level> Bids { get { return (ObservableCollection<Level>)this.Bids; } }
        //public ObservableCollection<Level> Asks { get { return (ObservableCollection<Level>)this.Asks; } }


        public Ticker Ticker
        {
            get { return this._ticker; } 
            set
            {
                this.Bids.Ticker = value;
                this.Asks.Ticker = value;
            }
        }

        #endregion Properties


        #region Interface implementation

        /*public event NotifyCollectionChangedEventHandler CollectionChanged;

        private void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
        {
            //marshal the event to the dispatcher's thread
            //System.Windows.Application.Current.Dispatcher.Invoke(DispatcherPriority.Send, new OnCollectionChangedHandler(OnCollectionChanged), e);
        }*/

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string name)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        public void NotifyDOMUpdated()
        {
            this.Dispatcher.Invoke(new Action(() =>
                {
                    OnPropertyChanged("Bids");
                    OnPropertyChanged("Asks");

                    this.Bids.RefreshSnapshot();
                    this.Asks.RefreshSnapshot();                    
                }));
        }

        public void NotifyBidsUpdated()
        {
            this.Dispatcher.BeginInvoke(new Action(() =>
            {
                OnPropertyChanged("Bids");
                this.Bids.RefreshSnapshot();
            }), null);
        }

        public void NotifyAsksUpdated()
        {
            this.Dispatcher.BeginInvoke(new Action(() =>
            {
                OnPropertyChanged("Asks");
                this.Asks.RefreshSnapshot();
            }), null);
        }

        
        #endregion Interface implementation
    }
}
